
//an animation
typedef struct anim_seq
{
	VECTOR frame_start;	//position of the first frame
	VECTOR frame_end;		//position of last frame
	VECTOR frame_size;	//x and y size of a frame
	var wait_delta;		//wait time 
	void* data;				//user data
}anim_seq;

//a sprite 
typedef struct sprite
{
	VECTOR force;
	var mass;
	ENTITY* ent;
	BMAP *sprt_sheet;		//sprite sheet  
	BMAP *disp_buf;		//the display buffer(bmap)
	//BMAP *bak_buf;	//a usefull back buffer(bmap)
	PANEL *panel;	//panel that holds bmap 
	char *name;		//user data
	void *data;		//user data
	int framesx;
}sprite;

//create a sprite
//bsource = bmap source of images for sprite
//bsize = the size of the sprite to create
//format = bmap format to create
sprite* sprite_create(BMAP* bsource,VECTOR *bsize,var format)
{
 	sprite *my_sprite=sys_malloc(sizeof(sprite));
 	if(my_sprite!=NULL)
 	{
 		if(bsource)
 		{
			my_sprite.sprt_sheet=bsource;
 			my_sprite.disp_buf=bmap_createblack(bsize.x,bsize.y,format);
 			if(my_sprite.disp_buf)
 			{
				my_sprite.panel=pan_create(NULL,0);
				if(my_sprite.panel)
				{
					my_sprite.panel.bmap=my_sprite.disp_buf;
					my_sprite.panel.center_x=bmap_width(my_sprite.disp_buf)/2;
					my_sprite.panel.center_y=bmap_height(my_sprite.disp_buf)/2;
					
					return my_sprite;
				}
 			}
 		}
 	}
 	printf("error create sprite");
 	return NULL;
}

//sprite show
void sprite_show(sprite* my_sprite)
{
	if(my_sprite)set(my_sprite.panel,SHOW);
}

//move sprite to position and layer
void sprite_move(sprite* my_sprite,VECTOR* pos)
{
	if(my_sprite)
	{
		my_sprite.panel.pos_x=pos.x;
		my_sprite.panel.pos_y=pos.y;
	}
}

//get sprite layer
var sprite_get_layer(sprite* my_sprite)
{
	if(my_sprite) return my_sprite.panel.layer;
	else return -1;
}

//blit a complete range of frames from a sequence of sprite_source into sprite_target
//seq = the animation seq to blit
//sprite = the sprite 
void sprite_anim_seq(sprite* my_sprite,anim_seq* seq)
{
	if(my_sprite && seq)
	{
		var cur;
		for(cur=seq.frame_start.x;cur<seq.frame_end.x;cur+=seq.frame_size.x)
		{
	  		bmap_blitpart(my_sprite.disp_buf,my_sprite.sprt_sheet,vector(0,0,0),vector(seq.frame_size.x,seq.frame_size.y,0),vector(cur,seq.frame_start.y,0),vector(seq.frame_size.x,seq.frame_size.y,0));
			wait(seq.wait_delta);
		}
	}
}


//blit one frame of sprite_source into sprite_target from range of frames defined by start-end
//sprite= the sprite
//seq = the animation sequence
//anim_frame = the frame to blit
void sprite_anim_frame(sprite* my_sprite,anim_seq* seq,int anim_frame)
{
	if(my_sprite!=NULL && seq!=NULL)
	{
		int anim_frames=(seq.frame_end.x-seq.frame_start.x)/seq.frame_size.x;
		var cur;
		if(anim_frame>anim_frames)anim_frame=0;
		if(anim_frame>0)cur=seq.frame_start.x+((seq.frame_size.x)*anim_frame);
		if(anim_frame==0)cur=0;
		if(anim_frame<0)cur=0;
  		bmap_blitpart(my_sprite.disp_buf,my_sprite.sprt_sheet,vector(0,0,0),vector(seq.frame_size.x,seq.frame_size.y,0),vector(cur,seq.frame_start.y,0),vector(seq.frame_size.x,seq.frame_size.y,0));
	}
}

BMAP* pix_buf;
//test for pixel collision
int pix_collide(sprite* sprite1,sprite* sprite2,int pixel_skip,int alpha_limit,VECTOR* collision_pos)
{
if(sprite1!=NULL && sprite1.panel.pos_x>0 && sprite1.panel.pos_x<screen_size.x-bmap_width(sprite1.disp_buf )
&& sprite1.panel.pos_y>0 && sprite1.panel.pos_y<screen_size.y-bmap_height(sprite1.disp_buf))
{
	//alpha_limit=clamp(alpha_limit,1,100);
	//pixel_skip=clamp(pixel_skip,1,99999);
	var format;
	var pixel1;
	var pixel2;
	var pixel3;
	
	//blit sprites to test buffer
//	bmap_blit(pix_buf,sprite2.disp_buf,vector(sprite2.panel.pos_x,sprite2.panel.pos_y,0),vector(bmap_width(sprite2.disp_buf),bmap_height(sprite2.disp_buf),0));
	
	int siz=bmap_width(sprite1.disp_buf)*bmap_height(sprite1.disp_buf);
	
	//x y positions on bmaps
	int a=0;int b=0;
	int count=0;
	for(count=0;count<siz;count+=pixel_skip)
	{
		//increase x position by pixel step
		a+=pixel_skip;
		
		//check bounds
		if(b>bmap_height(sprite1.disp_buf)-1){b=0;}
		if(a>bmap_width(sprite1.disp_buf)-1){a=0;b+=pixel_skip;}
		
		//lock sprite1
		format = bmap_lock(sprite1.disp_buf,0);
		//get pixel at x and y
		pixel1=pixel_for_bmap(sprite1.disp_buf,a,b);
		
		//get pixel data 
		COLOR pix_col1;
		var pix_alpha1;
		pixel_to_vec(pix_col1,pix_alpha1,format,pixel1);
		//unlock sprite1
		bmap_unlock(sprite1.disp_buf);
		
		//if the pixel is transparent then skip it
		if(pix_alpha1<alpha_limit)continue;
		
		//lock pixel buffer
		format = bmap_lock(sprite2.disp_buf,0);
		//get pixel from pixel buffer at x and y of sprite1 location
		pixel2=pixel_for_bmap(sprite2.disp_buf,sprite1.panel.pos_x+a,sprite1.panel.pos_y+b);
		
		//get pixel data
		COLOR pix_col2;
		var pix_alpha2;
		pixel_to_vec(pix_col2,pix_alpha2,format,pixel2);
		//unlock pixel buffer
		bmap_unlock(sprite2.disp_buf);
		
		//check collision
		if(pix_alpha2!=pix_alpha1)
		{
			//found a pixel intersecting sprite1 that does not belong
			//if the pixel is transparent then skip it
			if(pix_alpha2<alpha_limit)continue;
			
			//draw the pixel collision for debug 
			//pixel3 = pixel_for_vec(vector(0,0,255),100,bmap_lock(sprite1.disp_buf,0));
			//pixel_to_bmap(sprite1.disp_buf,a,b,pixel3);
			//bmap_unlock(sprite1.disp_buf);
			
			//collision=true
			collision_pos.x=sprite1.panel.pos_x+a;
			collision_pos.y=sprite1.panel.pos_y+b;
			collision_pos.z=0;
			return 1;
		}
	}
}	
	//collision=false
	return 0;
}

void remove_pixels_with(sprite* mask,sprite* targ)
{
	if(mask !=NULL && targ!=NULL)
	{
		var alpha_limit=1;
		var pixel_skip=1;
		var format;
		var pixel1;
		var pixel2;
		var pixel3;
		
		//blit sprites to test buffer
		bmap_blit(pix_buf,targ.disp_buf,vector(targ.panel.pos_x,targ.panel.pos_y,0),vector(bmap_width(targ.disp_buf),bmap_height(targ.disp_buf),0));
		
		int siz=bmap_width(mask.disp_buf)*bmap_height(mask.disp_buf);
		
		//x y positions on bmaps
		int a=0;int b=0;
		int count=0;
		for(count;count<siz;count+=pixel_skip)
		{
			//increase x position by pixel step
			a+=pixel_skip;
			
			//check bounds
			if(b>bmap_height(mask.disp_buf)){b=0;}
			if(a>bmap_width(mask.disp_buf)){a=0;b+=pixel_skip;}
			
			//lock sprite1
			format = bmap_lock(mask.disp_buf,0);
			//get pixel at x and y
			pixel1=pixel_for_bmap(mask.disp_buf,a,b);
			
			//get pixel data 
			COLOR pix_col1;
			var pix_alpha1;
			pixel_to_vec(pix_col1,pix_alpha1,format,pixel1);
			//unlock sprite1
			bmap_unlock(mask.disp_buf);
			
			//if the pixel is transparent then skip it
			if(pix_alpha1<alpha_limit)continue;
			
			//lock pixel buffer
			format = bmap_lock(pix_buf,0);
			//get pixel from pixel buffer at x and y of sprite1 location
			pixel2=pixel_for_bmap(pix_buf,mask.panel.pos_x+a,mask.panel.pos_y+b);
			
			//get pixel data
			COLOR pix_col2;
			var pix_alpha2;
			pixel_to_vec(pix_col2,pix_alpha2,format,pixel2);
			//unlock pixel buffer
			bmap_unlock(pix_buf);
			
			//check collision
			if(pix_alpha2!=pix_alpha1)
			{
				//found a pixel intersecting sprite1 that does not belong
				//if the pixel is transparent then skip it
				if(pix_alpha2<alpha_limit)continue;
				
				pixel3 = pixel_for_vec(vector(0,0,0),0,bmap_lock(targ.disp_buf,0));
				pixel_to_bmap(targ.disp_buf,mask.panel.pos_x+a,mask.panel.pos_y+b,pixel3);
				bmap_unlock(targ.disp_buf);
			}
		}
	}
}

int pix_collide_pos(sprite* sprite1,sprite* sprite2,VECTOR* ptarget,VECTOR* collision_pos)
{
if(sprite1!=NULL && sprite1.panel.pos_x>0 && sprite1.panel.pos_x<screen_size.x-bmap_width(sprite1.disp_buf )
&& sprite1.panel.pos_y>0 && sprite1.panel.pos_y<screen_size.y-bmap_height(sprite1.disp_buf))
{
	var format;
	var pixel1;
	var pixel2;
	
	//blit sprites to test buffer
	//bmap_blit(pix_buf,sprite2.disp_buf,vector(sprite2.panel.pos_x,sprite2.panel.pos_y,0),vector(bmap_width(sprite2.disp_buf),bmap_height(sprite2.disp_buf),0));
		
	//x y positions on bmaps
	int a=ptarget.x;int b=ptarget.y;
			
	//lock sprite1
	format = bmap_lock(sprite1.disp_buf,0);
	//get pixel at x and y
	pixel1=pixel_for_bmap(sprite1.disp_buf,a,b);
	
	//get pixel data 
	COLOR pix_col1;
	var pix_alpha1;
	pixel_to_vec(pix_col1,pix_alpha1,format,pixel1);
	//unlock sprite1
	bmap_unlock(sprite1.disp_buf);

	//lock pixel buffer
	format = bmap_lock(sprite2.disp_buf,0);
	//get pixel from pixel buffer at x and y of sprite1 location
	pixel2=pixel_for_bmap(sprite2.disp_buf,sprite1.panel.pos_x+a,sprite1.panel.pos_y+b);
	
	//get pixel data
	COLOR pix_col2;
	var pix_alpha2;
	pixel_to_vec(pix_col2,pix_alpha2,format,pixel2);
	//unlock pixel buffer
	bmap_unlock(sprite2.disp_buf);
//	bmap_unlock(sprite1.disp_buf);
	//check collision
	if(pix_alpha2!=pix_alpha1)
	{
		//found a pixel intersecting sprite1 that does not belong
		//if the pixel is transparent then skip it
		if(pix_alpha2==0)return 0;

		//collision=true
		collision_pos.x=sprite1.panel.pos_x+a;
		collision_pos.y=sprite1.panel.pos_y+b;
		collision_pos.z=0;
		return 1;
	}
}
	
	//collision=false
	return 0;
}

//create 2d terrain from noisemap 
BMAP* create_terr(char *noisefile,int noise_y,char *earth_texturefile,char *grass_texturefile,int grass_y,int mirror_cave)
{
	int px=0;
	int py=0;
	BMAP* pterrain=bmap_createblack(screen_size.x,screen_size.y,32);
	BMAP* noisemap=bmap_create(noisefile);
	BMAP* earth_texture=bmap_create(earth_texturefile);
	BMAP* grass_texture=bmap_create(grass_texturefile);
	
	if(pterrain!=NULL && noisemap!=NULL && earth_texture!=NULL && grass_texture!=NULL)
	{
		if(noise_y<0)noise_y=0;
		if(noise_y>bmap_height(noisemap)-1)noise_y=bmap_height(noisemap)-1;
		if(grass_y<0)grass_y=0;
		if(grass_y>bmap_height(grass_texture))grass_y=bmap_height(grass_texture);
		
		bmap_fill(pterrain,vector(0,0,0),0);
	
		///////////////////////////////////
		random_seed(0);
		//add apple parts at positions	
		BMAP* apple1="apple1.png";
		BMAP* apple2="apple2.png";
		BMAP* apple3="apple3.png";
		bmap_blit(pterrain,apple1,vector(bmap_width(pterrain)/2-random(bmap_width(apple1)),550,0),vector(0,0,0));
		bmap_blit(pterrain,apple2,vector(0+random(bmap_width(apple2)*3),550,0),vector(0,0,0));
		bmap_blit(pterrain,apple3,vector(bmap_width(pterrain)-random(bmap_width(apple3)*3),550,0),vector(0,0,0));
		///////////////////////////////////
		
		for(px=0;px<bmap_width(noisemap);px++)
		{
			//read noise map values
			VECTOR pix_col;
			var pix_alpha;
			var format=bmap_lock(noisemap,0);
			var pixel=pixel_for_bmap(noisemap,px,noise_y);
			pixel_to_vec(pix_col,pix_alpha,format,pixel);
			
			//values = red + green + blue
			int value1=pix_col.x+pix_col.y+pix_col.z;
	
			bmap_unlock(noisemap);
			
			int py;
			//create the terrain at height of noise color value
			for(py=0;py<0+value1/2;py++)
			{
				//copy pixel from earth texture
				format=bmap_lock(earth_texture,0);
				var pixel_in=pixel_for_bmap(earth_texture,px,bmap_height(earth_texture)-py);
				pixel_to_vec(pix_col,pix_alpha,format,pixel_in);
				bmap_unlock(earth_texture);
				
				//paste pixel onto terrain bmap
				format=bmap_lock(pterrain,0);
				pixel_to_bmap(pterrain,px,bmap_height(pterrain)-py,pixel_in);				
				bmap_unlock(pterrain);				
			}

			for(py=0;py<grass_y;py++)
			{			
				//copy pixel from grass texture
				format=bmap_lock(grass_texture,0);
				var pixel_in=pixel_for_bmap(grass_texture,px,(bmap_height(grass_texture)-1)-py);
				pixel_to_vec(pix_col,pix_alpha,format,pixel_in);
				bmap_unlock(grass_texture);
				
				//paste pixel onto terrain bmap
				format=bmap_lock(pterrain,0);
				pixel_to_bmap(pterrain,px,(bmap_height(pterrain)-(value1/2))-py,pixel_in);
				bmap_unlock(pterrain);				
			}

if(mirror_cave==1)
{
			for(py=0;py<0+value1/2;py++)
			{
				//copy pixel from earth texture
				format=bmap_lock(earth_texture,0);
				var pixel_in=pixel_for_bmap(earth_texture,px,bmap_height(earth_texture)-py);
				pixel_to_vec(pix_col,pix_alpha,format,pixel_in);
				bmap_unlock(earth_texture);
				
				//paste pixel onto terrain bmap
				format=bmap_lock(pterrain,0);
				pixel_to_bmap(pterrain,px,0+py,pixel_in);				
				bmap_unlock(pterrain);				
			}
		 }
}
		 //hand over the new terrain
		return pterrain ;
	}
	return NULL;
}